let trainRawEndModels = ModelManager.loadPartedRawModel(
    Resources.manager(),
    Resources.id(train_models[0]),
    null
);
let trainRawMiddleModels = ModelManager.loadPartedRawModel(
    Resources.manager(),
    Resources.id(train_models[1]),
    null
);

let trainEndModels = uploadPartedTrainModels(trainRawEndModels);
let trainMiddleModels = uploadPartedTrainModels(trainRawMiddleModels);

create_queue.push(function (ctx, state, train) {});

dispose_queue.push(function (ctx, state, train) {});

render_queue.push(function (ctx, state, train) {
    var matrices = new Matrices();

    for (let i = 0; i < train.trainCars(); i++) {
        function drawCarModel(model, car, matrices) {
            if (model === undefined) return; // Simple check if model doesn't exist

            ctx.drawCarModel(model, car, matrices);
        }

        function drawMultipleModels(models, car, matrices, modelNames) {
            modelNames.forEach((modelName) =>
                drawCarModel(models[modelName], car, matrices)
            );
        }

        matrices.pushPose();
        let models;
        if (i == 0) {
            models = trainEndModels;
        } else if (i == train.trainCars() - 1) {
            matrices.rotateY(Math.PI);
            models = trainEndModels;
        } else {
            models = trainMiddleModels;
        }
        drawMultipleModels(models, i, matrices, [
            'EXT',
            'BOGEY',
            'INT',
            'LAMPS',
            'WINDOWS',
            'WINDOWS_INT',
        ]);

        if (i != train.trainCars() - 1) {
            if (train.doorRightOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['LIGHT_R'], i, matrices);
            }
            if (train.doorLeftOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['LIGHT_L'], i, matrices);
            }

            let doorZ =
                train_doors_type === 'plug_fast'
                    ? smoothEnds(0, 0.75, 0, 0.5, train.doorValue())
                    : smoothEnds(-0.75, 0.75, -0.5, 0.5, train.doorValue());
            let doorX =
                train_doors_type === 'plug_fast'
                    ? train.doorValue() < 0.05
                        ? -train.doorValue() * 20 - 0.01
                        : -1.01
                    : 0;
            let doorZL = train.doorLeftOpen[i] ? doorZ : 0;
            let doorZR = train.doorRightOpen[i] ? doorZ : 0;
            let doorXL = train.doorLeftOpen[i] ? doorX / 16 : 0;
            let doorXR = train.doorRightOpen[i] ? doorX / 16 : 0;

            matrices.pushPose();
            matrices.translate(doorXR, 0, -doorZR);
            drawMultipleModels(models, i, matrices, [
                'D_LR_EXT',
                'D_LR_INT',
                'D_LR_WIN',
            ]);
            if (train.doorRightOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_LR_LIGHT'], i, matrices);
            }
            matrices.popPushPose();
            matrices.translate(doorXR, 0, doorZR);
            drawMultipleModels(models, i, matrices, [
                'D_LL_EXT',
                'D_LL_INT',
                'D_LL_WIN',
            ]);
            if (train.doorRightOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_LL_LIGHT'], i, matrices);
            }
            matrices.popPushPose();
            matrices.translate(-doorXL, 0, -doorZL);
            drawMultipleModels(models, i, matrices, [
                'D_RL_EXT',
                'D_RL_INT',
                'D_RL_WIN',
            ]);
            if (train.doorLeftOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_RL_LIGHT'], i, matrices);
            }
            matrices.popPushPose();
            matrices.translate(-doorXL, 0, doorZL);
            drawMultipleModels(models, i, matrices, [
                'D_RR_EXT',
                'D_RR_INT',
                'D_RR_WIN',
            ]);
            if (train.doorLeftOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_RR_LIGHT'], i, matrices);
            }
            matrices.popPose();
        } else {
            if (train.doorLeftOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['LIGHT_R'], i, matrices);
            }
            if (train.doorRightOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['LIGHT_L'], i, matrices);
            }

            let doorZ =
                train_doors_type === 'plug_fast'
                    ? smoothEnds(0, 0.75, 0, 0.5, train.doorValue())
                    : smoothEnds(-0.75, 0.75, -0.5, 0.5, train.doorValue());
            let doorX =
                train_doors_type === 'plug_fast'
                    ? train.doorValue() < 0.05
                        ? -train.doorValue() * 20 - 0.01
                        : -1.01
                    : 0;
            let doorZR = train.doorLeftOpen[i] ? doorZ : 0;
            let doorZL = train.doorRightOpen[i] ? doorZ : 0;
            let doorXR = train.doorLeftOpen[i] ? doorX / 16 : 0;
            let doorXL = train.doorRightOpen[i] ? doorX / 16 : 0;

            matrices.pushPose();
            matrices.translate(doorXR, 0, -doorZR);
            drawMultipleModels(models, i, matrices, [
                'D_LR_EXT',
                'D_LR_INT',
                'D_LR_WIN',
            ]);
            if (train.doorLeftOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_LR_LIGHT'], i, matrices);
            }
            matrices.popPushPose();
            matrices.translate(doorXR, 0, doorZR);
            drawMultipleModels(models, i, matrices, [
                'D_LL_EXT',
                'D_LL_INT',
                'D_LL_WIN',
            ]);
            if (train.doorLeftOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_LL_LIGHT'], i, matrices);
            }
            matrices.popPushPose();
            matrices.translate(doorXL, 0, -doorZL);
            drawMultipleModels(models, i, matrices, [
                'D_RL_EXT',
                'D_RL_INT',
                'D_RL_WIN',
            ]);
            if (train.doorRightOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_RL_LIGHT'], i, matrices);
            }
            matrices.popPushPose();
            matrices.translate(doorXL, 0, doorZL);
            drawMultipleModels(models, i, matrices, [
                'D_RR_EXT',
                'D_RR_INT',
                'D_RR_WIN',
            ]);
            if (train.doorRightOpen[i] && train.doorValue() > 0) {
                drawCarModel(models['D_RR_LIGHT'], i, matrices);
            }
            matrices.popPose();
        }

        matrices.popPose();
    }
});

function uploadPartedTrainModels(rawModels) {
    result = {};
    for (it = rawModels.entrySet().iterator(); it.hasNext(); ) {
        entry = it.next();
        if (['FARY', 'BUF'].includes('' + entry.getKey())) continue;
        if (
            [
                'D_RR_EXT',
                'D_LL_EXT',
                'D_LL_EXT',
                'D_LR_EXT',
                'INT',
                'D_RL_EXT',
            ].includes('' + entry.getKey())
        ) {
            entry.getValue().setAllRenderType('interior');
        } else if (
            [
                'LIGHT_R',
                'LIGHT_L',
                'D_RL_LIGHT',
                'D_RR_LIGHT',
                'D_LR_LIGHT',
                'D_LL_LIGHT',
                'LAMPS',
            ].includes('' + entry.getKey())
        ) {
            entry.getValue().setAllRenderType('lighttranslucent');
        } else if (
            [
                'WINDOWS',
                'WINDOWS_INT',
                'D_RR_WIN',
                'D_LR_WIN',
                'D_RL_WIN',
                'D_LL_WIN',
            ].includes('' + entry.getKey())
        ) {
            entry.getValue().setAllRenderType('interiortranslucent');
        }
        entry.getValue().applyUVMirror(false, true);
        result[entry.getKey()] = ModelManager.uploadVertArrays(
            entry.getValue()
        );
        if (typeof train_replace_textures !== 'undefined') {
            Object.entries(train_replace_textures).forEach(
                ([originalTexture, newTexture]) => {
                    result[entry.getKey()] =
                        result[entry.getKey()].copyForMaterialChanges();
                    result[entry.getKey()].replaceTexture(
                        originalTexture,
                        Resources.id(newTexture)
                    );
                }
            );
        }
    }
    return result;
}

// Анимация дверей, скопированная из MTR
function smoothEnds(startValue, endValue, startTime, endTime, time) {
    if (time < startTime) return startValue;
    if (time > endTime) return endValue;
    let timeChange = endTime - startTime;
    let valueChange = endValue - startValue;
    return (
        (valueChange *
            (1 - Math.cos((Math.PI * (time - startTime)) / timeChange))) /
            2 +
        startValue
    );
}
