importPackage(java.awt);
importPackage(java.awt.geom);
include(Resources.id('mtrsteamloco:scripts/display_helper.js'));

let slotCfg = {
    version: 1,
    texSize: [1024, 128],
    slots: [
        {
            name: 'head',
            texArea: [10, 0, 1024, 128],
            pos: [
                [
                    [-1.0636, 2.0779, 10.198],
                    [-1.0636, 1.8462, 10.198],
                    [0.3366, 1.8462, 10.198],
                    [0.3366, 2.0779, 10.198],
                ],
            ],
            offsets: [[0, 0, 0]],
        },
    ],
};

let slotReversedCfg = {
    version: 1,
    texSize: [1024, 128],
    slots: [
        {
            name: 'head-reversed',
            texArea: [10, 0, 1024, 128],
            pos: [
                [
                    [1.0636, 2.0779, -10.198],
                    [1.0636, 1.8462, -10.198],
                    [-0.3366, 1.8462, -10.198],
                    [-0.3366, 2.0779, -10.198],
                ],
            ],
            offsets: [[0, 0, 0]],
        },
    ],
};

let dhBase = new DisplayHelper(slotCfg);
let dhReversedBase = new DisplayHelper(slotReversedCfg);

function create(ctx, state, train) {
    state.pisRateLimit = new RateLimit(0.05);
    state.dh = dhBase.create();
    state.dhReversed = dhReversedBase.create();
    state.randomID = Math.floor(1 + (Math.random() * 99));							//Генерация рандомного номера маршрута (От 1 до 30)
	if (state.randomID >9) return (state.routeID = "0"+state.randomID);			    //Добавление нуля перед сгенерированным номером на ЭМУ
	else return (state.routeID = "00"+state.randomID);								//Добавление двух нулей в случае если число меньше 10
}

function dispose(ctx, state, train) {
    state.dh.close();
    state.dhReversed.close();
}

var EMUFont = Resources.readFont(
    Resources.idRelative('mtr:metropackint/fonts/nomer_6.ttf')
);

function render(ctx, state, train) {
    if (state.pisRateLimit.shouldUpdate()) {
        let g;

        let EMUText = getDestinationStation(train);

        [
            { dh: state.dh, graphicsName: 'head' },
            { dh: state.dhReversed, graphicsName: 'head-reversed' },
        ].forEach((element) => {
            let dh = element.dh;
            g = dh.graphicsFor(element.graphicsName);
            g.setColor(Color.BLACK);
            g.fillRect(0, 0, 1024, 128);
            g.setColor(Color.ORANGE);
            g.setFont(EMUFont.deriveFont(Font.PLAIN, 62));
            let fontMetrics = g.getFontMetrics();
            let routeWidth = fontMetrics.stringWidth(state.routeID);
            let textWidth = fontMetrics.stringWidth(EMUText);
            g.drawString(
                EMUText,
                (1024 - routeWidth - textWidth) / 3,
                98
            );
            dh.upload();
        });
    }

    ctx.drawCarModel(state.dh.model, 0, null);
    ctx.drawCarModel(state.dhReversed.model, train.trainCars() - 1, null);
}

function getDestinationStation(train) {
    let stationList = train.getThisRoutePlatforms();
    let lastPlatform = stationList[stationList.size() - 1];
    if (lastPlatform === undefined) return 'ПОСАДКИ НЕТ';
    return lastPlatform.destinationName.split('\\|')[0].toUpperCase();
}
